searchData={"items":[{"type":"module","title":"CozyParams","doc":"Expose more user friendly API for underlying modules.","ref":"CozyParams.html"},{"type":"macro","title":"CozyParams.defparams/2","doc":"Defines a function for casting and validating params.\n\n> Essentially, this macro is just a shortcut for using `CozyParams.Schema`.\n\nInspired by [vic/params](https://github.com/vic/params).","ref":"CozyParams.html#defparams/2"},{"type":"macro","title":"Examples - CozyParams.defparams/2","doc":"```elixir\ndefmodule Demo do\n  import CozyParams\n\n  defparams :product_search do\n    field :name, :string, required: true\n  end\n\n  def search(params) do\n    with {:ok, data} <- product_search(params) do\n      # process data\n    end\n  end\nend\n```\n\nAbove `defparams :product_search do: block` will:\n1. create a module `Demo.CozyParams.ProductSearch` automatically.\n2. inject `product_search/1` into current module. And, this function\n   will call `Demo.CozyParams.ProductSearch.from/1` internally.\n\nFor more details of the schema definitions in `do: block`, check out `CozyParams.Schema`.","ref":"CozyParams.html#defparams/2-examples"},{"type":"macro","title":"Error handling - CozyParams.defparams/2","doc":"When external params are invalid, `{:error, params_changeset: %Ecto.Changeset{}}`\nwill be returned, which allows developers to match this pattern for handling errors.\n\nIf the error messages is required, `CozyParams.get_error_messages/1` would be helpful.","ref":"CozyParams.html#defparams/2-error-handling"},{"type":"function","title":"CozyParams.get_error_messages/1","doc":"Extract error messages from `%Ecto.Changeset{}`.","ref":"CozyParams.html#get_error_messages/1"},{"type":"function","title":"CozyParams.get_error_messages/2","doc":"Extract error messages from `%Ecto.Changeset{}` with a given function.\n\nThe `msg_func` will be passed to `Ecto.Changeset.traverse_errors/2`, read doc of\n`Ecto.Changeset.traverse_errors/2` for more information.","ref":"CozyParams.html#get_error_messages/2"},{"type":"module","title":"CozyParams.Changeset","doc":"Changeset tailored for this lib.","ref":"CozyParams.Changeset.html"},{"type":"module","title":"CozyParams.Schema","doc":"Provides macros for defining schemas which is used for casting and validating params.\n\n> Due to limitations of implementation, it's impossible to document every supported\n> macros within `@doc`. Because of that, I will try to best to document them\n> within `@moduledoc`.","ref":"CozyParams.Schema.html"},{"type":"module","title":"Available macros - CozyParams.Schema","doc":"`CozyParams.Schema` provides a subset of macros supported by Ecto, In addition,\nit makes some changes on `embeds_one` and `embeds_many`.\n\nIn a nutshell, all available macros are:\n\n1. `schema(do: block)`\n2. `field(name, type, opts \\\\ [])`\n   - available `opts`:\n     - `:default`\n     - `:required` - default: `false`\n     - `:pre_cast` - specify a unary function to process data before casting\n3. `embeds_one(name, opts \\\\ [], do: block)`\n   - available `opts`:\n     - `:required` - default: `false`\n4. `embeds_one(name, schema, opts \\\\ [])`\n   - available `opts`:\n     - `:required` - default: `false`\n5. `embeds_many(name, opts \\\\ [], do: block)`\n   - available `opts`:\n     - `:required` - default: `false`\n6. `embeds_many(name, schema, opts \\\\ [])`\n   - available `opts`:\n     - `:required` - default: `false`\n\n> `type` argument will be passed to `Ecto.Schema`, so it supports all the types supported\n> by `Ecto.Schema` in theory.\n> Read *Types and casting* section of `Ecto.Schema` to get more information.","ref":"CozyParams.Schema.html#module-available-macros"},{"type":"module","title":"Examples - CozyParams.Schema","doc":"There're 2 ways to define a schema:\n1. use inline syntax\n2. use extra modules\n\n#","ref":"CozyParams.Schema.html#module-examples"},{"type":"module","title":"Define a schema contains embedded fields in inline syntax: - CozyParams.Schema","doc":"```elixir\ndefmodule PersonParams do\n  use CozyParams.Schema\n\n  schema do\n    field :name, :string, required: true\n    field :age, :integer\n    field :wishlist, {:array, :string}, pre_cast: &String.split(&1, ~r/,\\s*/)\n\n    embeds_one :mate, required: true do\n      field :name, :string, required: true\n      field :age, :integer\n    end\n\n    embeds_many :pets do\n      field :name, :string, required: true\n      field :breed, :string\n    end\n  end\nend\n\nPersonParams.from(%{\n  name: \"Charlie\",\n  mate: %{\n    name: \"Lucy\"\n  },\n  wishlist: \"table,chair,computer\"\n})\n```\n\n\n#","ref":"CozyParams.Schema.html#module-define-a-schema-contains-embedded-fields-in-inline-syntax"},{"type":"module","title":"Define a schema contains embedded fields with extra modules: - CozyParams.Schema","doc":"```elixir\ndefmodule PersonParams do\n  use CozyParams.Schema\n\n  defmodule Mate do\n    use CozyParams.Schema\n\n    schema do\n      field :name, :string, required: true\n      field :age, :integer\n      field :wishlist, {:array, :string}, pre_cast: &String.split(&1, ~r/,\\s*/)\n    end\n  end\n\n  defmodule Pet do\n    use CozyParams.Schema\n\n    schema do\n      field :name, :string, required: true\n      field :breed, :string\n    end\n  end\n\n  schema do\n    field :name, :string, required: true\n    field :age, :integer\n    embeds_one :mate, Mate, required: true\n    embeds_many :pets, Pet\n  end\nend\n\nPersonParams.from(%{\n  name: \"Charlie\",\n  mate: %{\n    name: \"Lucy\"\n  },\n  wishlist: \"table,chair,computer\"\n})\n```","ref":"CozyParams.Schema.html#module-define-a-schema-contains-embedded-fields-with-extra-modules"},{"type":"module","title":"About the generated functions - CozyParams.Schema","doc":"`schema/1` will create 2 functions automatically:\n\n1. `changeset/2` which is overridable.\n2. `from/1` which will be called by high-level abstractions, such as `CozyParams`.","ref":"CozyParams.Schema.html#module-about-the-generated-functions"},{"type":"module","title":"Reflection - CozyParams.Schema","doc":"Schemas will generate the following functions that can be used for runtime\nintrospection of the schema:\n\n+ `__cozy_params_schema__/0`\n+ `__cozy_params_schema__/1`\n+ `__cozy_params_changeset__/0`\n+ `__cozy_params_changeset__/1`\n\nAll possible calls:\n\n+ `__cozy_params_schema__()` - an alias of `__cozy_params_schema__(:metadata)`.\n+ `__cozy_params_schema__(:metadata)` - returns the metadata of current schema.\n+ `__cozy_params_schema__(:original_ast)` - returns the original AST passed to `schema/1`.\n+ `__cozy_params_schema__(:transpiled_ast)` - returns the transpiled AST used by `cozy_params`.\n+ `__cozy_params_schema__(:ecto_ast)` - returns the AST used by Ecto.\n+ `__cozy_params_changeset__()` - an alias of `__cozy_params_changeset__(:metadata)`.\n+ `__cozy_params_changeset__(:metadata)` - returns the metadata used by `CozyParams.Changeset`.","ref":"CozyParams.Schema.html#module-reflection"},{"type":"macro","title":"CozyParams.Schema.schema/1","doc":"","ref":"CozyParams.Schema.html#schema/1"},{"type":"module","title":"CozyParams.Schema.AST","doc":"Process the AST of `cozy_params`.","ref":"CozyParams.Schema.AST.html"},{"type":"function","title":"CozyParams.Schema.AST.as_block/1","doc":"Make sure that the AST is always a block.","ref":"CozyParams.Schema.AST.html#as_block/1"},{"type":"function","title":"CozyParams.Schema.AST.as_ecto_block/1","doc":"Transform `cozy_params` supported AST to Ecto supported AST.\n\nCurrently, this function will:\n+ drop options which are supported by `cozy_params` only. Or, Ecto will report\n  invalid option errors.","ref":"CozyParams.Schema.AST.html#as_ecto_block/1"},{"type":"function","title":"CozyParams.Schema.AST.extract_metadata/1","doc":"Extract metadata from AST.","ref":"CozyParams.Schema.AST.html#extract_metadata/1"},{"type":"function","title":"CozyParams.Schema.AST.transpile_block/2","doc":"Transpile shortcuts of `embeds_one` and `embeds_many`.\n\n`cozy_params` will transpile code like following one:\n\n```elixir\nembeds_one :address, required: true do\n  field :latitude, :float, required: true\n  field :longtitude, :float, required: true\nend\n```\n\nto Ecto supported code:\n\n```elixir\ndefmodule Address do\n  use CozyParams.Schema\n\n  schema do\n    field :latitude, :float, required: true\n    field :longtitude, :float, required: true\n  end\nend\n\nembeds_one :address, Address\n```","ref":"CozyParams.Schema.AST.html#transpile_block/2"},{"type":"function","title":"CozyParams.Schema.AST.validate_block!/1","doc":"Validate if the block meets `cozy_params`'s requirements.\n\nAlthough `cozy_params` try its best to emulate the API of `Ecto.Schema`, but it\ndoesn't mean that every API of `Ecto.Schema` is supported by `cozy_params`.\n\nBecause of that, `cozy_params` validates the calls before proceeding.","ref":"CozyParams.Schema.AST.html#validate_block!/1"},{"type":"extras","title":"CozyParams","doc":"# CozyParams\n\n[![CI](https://github.com/cozy-elixir/cozy_params/actions/workflows/ci.yml/badge.svg)](https://github.com/cozy-elixir/cozy_params/actions/workflows/ci.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/cozy_params.svg)](https://hex.pm/packages/cozy_params)\n[![built with Nix](https://img.shields.io/badge/built%20with%20Nix-5277C3?logo=nixos&logoColor=white)](https://builtwithnix.org)\n\nProvides Ecto-like API for casting and validating params.","ref":"readme.html"},{"type":"extras","title":"Why another package? - CozyParams","doc":"There are some packages in the community:\n\n| NAME                                                      | Based on Ecto? | Use Ecto-like API? |\n| --------------------------------------------------------- | -------------- | ------------------ |\n| [params](https://github.com/vic/params)                   | YES            | NO                 |\n| [maru_params](https://github.com/elixir-maru/maru_params) | NO             | NO                 |\n| [tarams](https://github.com/bluzky/tarams)                | NO             | NO                 |\n| ...                                                       |                |                    |\n\nBut, they don't fit my requirements. The package in my dream should:\n\n- be based on [Ecto](https://github.com/elixir-ecto/ecto) which is robust and battle-tested.\n- use Ecto-like API, which eliminates friction when working on casting params and modeling data source at the same time.","ref":"readme.html#why-another-package"},{"type":"extras","title":"Installation - CozyParams","doc":"Add `:cozy_params` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:cozy_params, \" \"}\n  ]\nend\n```\n\n(Optional) And, it's encouraged to setup `.formatter.exs` for `cozy_params`:\n\n```elixir\n[\n  import_deps: [\n    # ...\n    :cozy_params\n  ],\n  # ...\n]\n```","ref":"readme.html#installation"},{"type":"extras","title":"Overview - CozyParams","doc":"- `CozyParams` - provides macros for general usage.\n- `CozyParams.Schema` - provides macros for defining schemas directly.\n\nAn example integrating with Phoenix:\n\n```elixir\ndefmodule DemoWeb.PageController do\n  use DemoWeb, :controller\n  import CozyParams\n\n  action_fallback DemoWeb.FallbackController\n\n  defparams :product_search do\n    field :name, :string, required: true\n\n    embeds_many :tags do\n      field :name, :string, required: true\n    end\n  end\n\n  def index(conn, params) do\n    with {:ok, data} <- product_search(params) do\n      # ...\n    end\n  end\nend\n\ndefmodule DemoWeb.FallbackController do\n  use DemoWeb, :controller\n\n  # ...\n\n  # handle errors for cozy_params\n  def call(conn, {:error, params_changeset: %Ecto.Changeset{} = changeset}) do\n    messages = CozyParams.get_error_messages(changeset)\n    # render messages in HTML, JSON, etc.\n  end\n\n  # handle errors for normal changsets from Ecto.\n  def call(conn, {:error, %Ecto.Changeset{} = changeset}) do\n    # ...\n  end\n\n  # ...\nend\n```\n\nVisit [HexDocs](https://hexdocs.pm/cozy_params) for more details.","ref":"readme.html#overview"},{"type":"extras","title":"License - CozyParams","doc":"Apache License 2.0","ref":"readme.html#license"},{"type":"extras","title":"FAQ","doc":"# FAQ","ref":"faq.html"},{"type":"extras","title":"How to integrate with Phoenix? - FAQ","doc":"#","ref":"faq.html#how-to-integrate-with-phoenix"},{"type":"extras","title":"casting and validating params - FAQ","doc":"```elixir\ndefmodule DemoWeb.PageController do\n  use DemoWeb, :controller\n  import CozyParams\n\n  defparams :product_search do\n    field :name, :string, required: true\n  end\n\n  def index(conn, params) do\n    with {:ok, data} <- product_search(params) do\n      # when external params are invalid, `{:error, params_changeset: %Ecto.Changeset{}}`\n      # will be returned.\n    end\n  end\nend\n```\n\n#","ref":"faq.html#casting-and-validating-params"},{"type":"extras","title":"error handling - FAQ","doc":"As mentioned above, when external params are invalid, `{:error, params_changeset: %Ecto.Changeset{}}` will be returned, which allows developers to match this pattern in the fallback controller, and convert the changeset as error messages by using `CozyParams.get_error_messages/1`.\n\nFor example:\n\n```elixir\ndefmodule DemoWeb.PageController do\n  use DemoWeb, :controller\n  import CozyParams\n\n  action_fallback DemoWeb.FallbackController\n\n  defparams :product_search do\n    field :name, :string, required: true\n  end\n\n  def index(conn, params) do\n    with {:ok, data} <- product_search(params) do\n      # ...\n    end\n  end\nend\n\ndefmodule DemoWeb.FallbackController do\n  use DemoWeb, :controller\n\n  # ...\n\n  # handle errors for cozy_params\n  def call(conn, {:error, params_changeset: %Ecto.Changeset{} = changeset}) do\n    messages = CozyParams.get_error_messages(changeset)\n    # render messages in HTML, JSON, etc.\n  end\n\n  # handle errors for normal changsets from Ecto.\n  def call(conn, {:error, %Ecto.Changeset{} = changeset}) do\n    # ...\n  end\n\n  # ...\nend\n```","ref":"faq.html#error-handling"},{"type":"extras","title":"How to expand the capabilities of schemas? - FAQ","doc":"By default, `cozy_params` just:\n\n- casts types of fields.\n- validates presence of fields.\n\nBut, people usually expect more, such as validating format, etc.\n\nThe easiest way is to override `changeset/2` in a schema. For example:\n\n```elixir\ndefmodule SampleParams do\n  use CozyParams.Schema\n\n  schema do\n    field :name, :string, required: true\n    field :age, :integer\n\n    embeds_one :mate, required: true do\n      field :name, :string, required: true\n      field :age, :integer\n    end\n\n    embeds_many :pets do\n      field :name, :string, required: true\n      field :breed, :string\n    end\n  end\n\n  def changeset(struct, params) do\n    struct\n    |> super(params)\n    |> Ecto.Changeset.validate_* # ...\n  end\nend\n```\n\nA harder way is to extend `CozyParams.Schema` at AST level, which requires you to improve `CozyParams.Schema.AST`.","ref":"faq.html#how-to-expand-the-capabilities-of-schemas"}],"content_type":"text/markdown"}